(function ($) {
	$(document).ready(function() {
		let chartRequest = {};
		let dash;
		let chart;
		let data;
		const $btnDateFilter = $('#btn-date-filter');
		$btnDateFilter.on('click', createChart);

		function createChart() {
			chartRequest = {
				fetchData: () => {
		   			return $.ajax({
		   				url: "sites/all/themes/lrebootstrap/scripts/php/view/results.php",
							data: {
								table: "dashboard.data_viewer_elev_graph_step2",
								fields: ["*"],
								drupal_user_switch: false,
								drupal_userid: 1,
								startDate: startDatePicker.getDate(),
								endDate: moment(endDatePicker.getDate()).add(1, 'days').format('YYYY-MM-DD') 
							}
		   			})
				},
				processData: (data) => {
					// first format date so date comparisons across can be made across stations
					for (var i = 0; i < data.length; i++) {
						data[i].date_time_formatted = parseDateTime(data[i].date_time);
					}

					var series  = unique(data); // return a list of distinct values for each field
					var locationName = series[2].values; // return a list of distinct location names present in the data
					var locations = series[1].values; // return a list of distinct location_ndx present in the data
					//var wellElevation = series[6].values; // return a list of distinct  measure types
					var locationCount = locations.length; // return the count of locations in the data
					var dates = series[7].values; // return a list of distinct dates present in the data

					// create empty arrays to for intermediate restructuring of data
					var dataArray = [];
					var datesArray = [];

					// generating an array of data for each station present in the data
					// and pushing to dataArray
					(function step1() {
						for (var i = 0; i < locations.length; i++) {
							var seriesArray = [];
							var s = locations[i];

							for (var j = 0; j < data.length; j++) {
								var d = data[j];
								if ( s == d.location_ndx) {
									d.date_time_formatted = parseDateTime(d.date_time);
									seriesArray.push(d);
								}
							}

							dataArray.push(seriesArray);
						}
						step2();
					})();

					// populate the dates array
					// create a record for each unqiue date
					// with a value and unit for each station present in the data
					function step2() {
						// loop through unique list of dates
						// creating properties for the original date and the formatted date
						for (var i = 0; i < dates.length; i++) {
							var recordObj = {
								date_time_formatted: dates[i],
								date_time: dates[i],
							}

							// loop through the locationCount
							// and create a value and unit field for each station present in the data
							for (var j = 0; j < locationCount; j++) {
								recordObj["value" + j] = null;
								//recordObj["unit" + j] = null;
								//recordObj["measureType" + j] = null;
							}

							// push generated record to the datesArray
							datesArray.push(recordObj);
						}
						step3();
					}

					// populate the value and unit field in the datesArray if a value present for that date
					function step3() {
						for (var i = 0; i < datesArray.length; i++) {
							for (var j = 0; j < locationCount; j++) {
								for (var ii = 0; ii < dataArray[j].length; ii++) {
									if (datesArray[i].date_time_formatted == dataArray[j][ii].date_time_formatted) {
										datesArray[i]["value" + j] = dataArray[j][ii].depth_to_gw_ft;
										//datesArray[i]['unit' + j] = dataArray[j][ii].unit_desc;
										//datesArray[i]['measureType' + j] = dataArray[j][ii].measure_type_desc;
										datesArray[i].location_desc = dataArray[j][ii].location_desc;
									}
								}
							}
						}
					}

					// set the data property equal to the datesArray after step3 runs
					chartRequest.data = datesArray;
				},
				setChipsData: (data) => {
					const chipsData = [...new Set(data.map(d => d.location_desc))];
					return chipsData
						.filter(d => d !== 'Pits' && d !== 'Well 6')
						.map((d, i) => (
						{ value: i, display: d }
					));
				},
				setData: () => {
					$.when(chartRequest.fetchData()).done(function(data) {
						var response = JSON.parse(data);
						if (response) {
							seriesToggle.data = chartRequest.setChipsData(response);
							seriesToggle.buildChips().selectAll();
							chartRequest.processData(response);
							google.charts.load('current', {'packages':['corechart', 'controls', 'gauge']});
							google.charts.setOnLoadCallback(drawChart);
							//chart = new Chart(chartOptions, chartRequest.data);
						}
					})
				}
			} // chartRequest			
		   	chartRequest.setData();
		} // createChart()


		var startDatePicker = new datePicker({
			elementExists: true,
			container:"#js-start-date",
		}).setDate(20);

		var endDatePicker = new datePicker({
			elementExists: true,
			container:"#js-end-date",
		}).setDate();

		var seriesToggle = new chipBag({
			parentContainer: '#chips-filter',
			elementExists: false,
			chipActiveColor: '#2196F3',
			data: [],
			onChange: () => {
				// data.setColumnProperty(1, );
				// var view = new google.visualization.DataView(data); //datatable contains col and rows
				// view.setColumns([0,1,3,4]); //only show these column
				// dash.draw(view); //pass the view to draw chat
			}
		});

		createChart();

		var chartTitle;

		function drawChart() {
			var request = chartRequest.data
			data  = new google.visualization.DataTable();
			data.addColumn('date', "date_time");
			data.addColumn('number', "value1");
			data.addColumn('number', "value2");
			data.addColumn('number', "value3");
			data.addColumn('number', "value4");
			data.addColumn('number', "value5");
			data.addColumn('number', "value6");

			// convert the date/time to a javascript date using the column index
			// for the date/time field
			for (var i = 0; i < request.length; i++) {
				request[i].date_time_formatted = new Date(request[i].date_time);
				data.addRows([[request[i].date_time_formatted, processNull(request[i].value0),processNull(request[i].value1),processNull(request[i].value2),processNull(request[i].value3),processNull(request[i].value4),processNull(request[i].value5)]]);
			}

			data.sort([{column: 0, desc:true}]);
			data.setColumnLabel(0,'Date/Time');
			data.setColumnLabel(1,'Well 1 Depth');
			data.setColumnLabel(2,'Well 2 Depth');
			data.setColumnLabel(3,'Well 3 Depth');
			data.setColumnLabel(4,'Well 4 Depth');
			data.setColumnLabel(5,'Well 5 Depth');
			data.setColumnLabel(6,'Well 6 Depth');
			var dateformatter = new google.visualization.DateFormat({pattern: "MMM d, h:mm a"});
			dateformatter.format(data, 0);

			// Create the dashboard
			dash = new google.visualization.Dashboard(document.getElementById('dash'));

			// Define a data filter control
			var zoomer = new google.visualization.ControlWrapper({
				controlType: 'ChartRangeFilter',
				containerId: 'zoomer',
				options: {
					width:'100%', 
					height: '100%',   
					chartArea: {
						left:40,
						width: '86%'
					},
					filterColumnIndex: 0,
					ui: {
						label: 'Select dates: ',
						labelStacking: 'horizontal',
						chartView: {columns:[0,2]},
						chartOptions: {
							series: { 
								0: {color: 'blue'}
							}
						}
					}
				}
			});

			var chartOptions = {
				chartType: 'ComboChart',
				containerId: 'chart',
				options: {  
					//bar: { groupWidth: "100%" }, 
				//title:"Well Water Level Elevations",
					height: 450, 
					width:'100%',
					chartArea: {
						left:65, 
						right:60,
					top:60, 
					bottom:60, 
					width: '83%', 
					backgroundColor: '#f5f5f5'
					},
					legend: {
						position: 'top', 
						alignment: 'center'
					},
								backgroundColor: {strokeWidth: 1},
					series: {
							0: {type: 'line', targetAxisIndex: 0, color: 'red', pointSize: 0, lineWidth: 3},
							1: {type: 'line', targetAxisIndex: 0, color: 'blue', pointSize: 0, lineWidth: 3},
							2: {type: 'line', targetAxisIndex: 0, color: 'green', pointSize: 0, lineWidth: 3},
							3: {type: 'line', targetAxisIndex: 0, color: 'orange', pointSize: 0, lineWidth: 3},
							4: {type: 'line', targetAxisIndex: 0, color: 'cyan', pointSize: 0, lineWidth: 3},
							5: {type: 'line', targetAxisIndex: 0, color: 'gray', pointSize: 0, lineWidth: 3},
							},
					vAxes: {
							0: {title:'Depth to Water, ft', format: '#.##'}
					},
					focusTarget: 'category'
				},
				//*****PUT COLUMN 6 BACK WHEN WELL 6 IS REPORTING PROPERLY****
				//    view: {'columns': [0,1,2,3,4,5,6]}
				view: {'columns': [0,1,2,3,4,5]}
			}

			// Define graph
			chart = new google.visualization.ChartWrapper(chartOptions);
			
			// function selectHandler() {
			// 	var col = chart.getChart().getSelection()[0].column;
			// 	var view = new google.visualization.DataView(data);
			// 	view.hideColumns([col]);
			// 	// chart.draw(view, options);
			// 	dash.bind([zoomer], [chart, table]);
			// 	dash.draw(view);
			// }

			// google.visualization.events.addListener(chart, 'select', selectHandler);

			var table= new google.visualization.ChartWrapper({
					containerId: 'table',
					chartType: 'Table',
					options: {
					height: '300px', 
					width: '100%'		                              
				},                              
				view: {'columns': [0, 1, 2, 3, 4, 5]}
			});


			// Establish dependencies
			dash.bind([zoomer], [chart, table]);
			dash.draw(data);
		}

	})
})(jQuery);