(function ($) {
	// Don't execute any of the below code until the document
	// Has fully loaded
	$( document ).ready(function() {

		// add map container class
		$( '.main-container' ).addClass('main-container-map-content');

		var popup = ''; // initialize popup

		// initialize button objects
		var $dateBtn = $( '#btn-date-filter' ),
			$locBtn = $( '.btn-loc-filter'),
			$hideBtn = $( '.btn-hide' );

		// initialize chart and chart filter objects
		var chart = {},
			chart2 = {},
			chart3 = {},
			chartOptions4 = {},
			startDatePicker = {},
			endDatePicker = {};

		// process Drupal settings exported from content type template
		// and create object to store sidebar filter settings
		var filterSettings = [{
			dataSource: Drupal.settings.dataSource1	
		}];

		// process Drupal settings exported from content type template
		// and create object to store data settings
		var dataSettings = {
    		titles: [Drupal.settings.dataDisplayTitle1, Drupal.settings.dataDisplayTitle2, Drupal.settings.dataDisplayTitle3],
    		visiblity: +Drupal.settings.dataDisplayVisibility
    	};

    	// process Drupal settings exported from content type template
		// and create object to store Mapbox map settings
		var mapSettings = {
			style: Drupal.settings.mapStyle,
			center: Drupal.settings.mapCenter,
			zoomControl: Drupal.settings.mapZoomControl,
			zoomLevel: +Drupal.settings.mapZoomLevel,
			selectedLocation: "2",
			selectedLocationName: "Well 2"
		}

		// when user applys data filters, redraw charts
		$dateBtn.on('click', createChart);

		// when user selects location, redraw charts
		$locBtn.on('click', createChart);
		$locBtn.on('click', updateDataTitles);
		$locBtn.on('click', maintenanceSettings);

		// hide data pane on button click
		$hideBtn.on('click', hidePane);

		// function used to show/hide data pane
		function hidePane() {		
			if (dataSettings.visiblity == 1) {
				$('#map').css('height', $(window).height());				
	    		setTimeout(function() { map.resize() }, 1000);
	    		dataSettings.visiblity == 0;
	    	}
		}

		// function used to update graph titles to incuded selected location
		function updateDataTitles() {
			// update data pane titles with selected location
	        var $dataDisplayTitle = $( '.data-display-title '),
	        	$serviceLogTitle = $( '#service-log-title' );

	        $.each($dataDisplayTitle, function(k,v) {
	        	var title = dataSettings.titles[k]
	        	$( this ).empty().html(title + ' (' + mapSettings.selectedLocationName + ')');
	        });

	        $serviceLogTitle.html("Last Service Log Entries for " + mapSettings.selectedLocationName);
		}

		// function used to generate maintenance settings
		function maintenanceSettings() {
			var d = [];
			function fetchData() {
    			return $.ajax({
    				url: "/sites/all/themes/lrebootstrap/scripts/php/view/results.php?",
	    			data: {
	    				table: "dashboard.maintenance_settings",
	    				fields: ["service_due_hrs","notification1_hrs","notification1_pct","notification2_hrs","notification2_pct"],
	    				filterField: Drupal.settings.dataFilterField,
	    				filterValue: mapSettings.selectedLocation,
	    				drupal_user_switch: false,
	    				drupal_userid: 1
	    			}
	    		})
    		}
			$.when(fetchData()).done(function(data) {
   				var response = JSON.parse(data);
   				if (response) {
   					d = response;
   					chartOptions4.gaugeRanges = response;
   					var $list = $('#maintenance-settings-list');
					var html = '<li>Service due after <strong>' + d[0].service_due_hrs + ' hours</strong>.</li><li>Notifications sent at <strong>'+ d[0].notification1_hrs + ' hrs (' + d[0].notification1_pct + '%)</strong> and <strong>' + d[0].notification2_hrs + ' hrs (' + d[0].notification2_pct + '%)</strong>.</li>';
					$list.empty().append(html);
   				}
   			})              
		}

		// function used to generate map popup
		function createPopup(feature) {
			var f = JSON.parse(feature.properties.most_recent_value);
			var date = new Date(feature.properties.most_recent_timestamp);
			var newDate = moment(date);
			var formattedDate = newDate.format("MM/DD/YY @ H:m A");
			var mostRecentHTML = '';

			for (var i = 0; i < f.length; i++) {
				var header = f[i].split(":")[0];
				var value = f[i].split(":")[1];

				mostRecentHTML += '<div class="col-sm-6"><div class="map-popup-subheader">'+header+'</div><div class="map-popup-box">'+value+'</div></div>';
			}

			popup = new mapboxgl.Popup({closeOnClick: false, closeButton: true})
     			.setLngLat(feature.geometry.coordinates)
     			.setHTML('<div class="map-popup-header-wrapper col-xs-12"><i class="material-icons md-24 text-white inline-block">location_on</i><div class="map-popup-header inline-block">'+feature.properties.location_desc+'</div></div><div class="clearfix"></div>' + mostRecentHTML)
     			.addTo(map);
		}

		// function used to create charts
		function createChart() {

			// configure chart options
	    	var chartOptions = {
	    		type: +Drupal.settings.dataDisplayType,
	    		container: "chart",
	    		fields: Drupal.settings.dataDisplayFields.split(","),
	    		fieldLabels: Drupal.settings.dataDisplayFieldLabels.split(","),
	    		fieldTypes: Drupal.settings.dataDisplayFieldTypes.split(","),
	    		table: {
	    			container: "service-log-table",
	    			data: []
	    		}
	    	};

	    	var tableRequest = {
	    		fetchData: function() {
	    			return $.ajax({
	    				url: "/sites/all/themes/lrebootstrap/scripts/php/view/results.php?",
		    			data: {
		    				table: "dashboard.service_log_processed",
		    				fields: ["date_time","station_svc_descrip"],
		    				filterField: Drupal.settings.dataFilterField,
		    				filterValue: mapSettings.selectedLocation,
		    				sort: "1 DESC", 
		    				limit: 3,
		    				drupal_user_switch: false,
		    				drupal_userid: 1
		    			}
		    		})
	    		},
	    		setData: function() {
	    			$.when(tableRequest.fetchData()).done(function(data) {
		   				var response = JSON.parse(data);
		   				if (response) {
		   					tableRequest.data = response;
	    					chartOptions.table.data = tableRequest.data;
		   				}
		   			})
	    		}
	    	}

	    	tableRequest.setData();

	    	// request chart data from database
	    	var chartRequest = {
	    		fetchData: function() {
	    			return $.ajax({
	    				url: "/sites/all/themes/lrebootstrap/scripts/php/view/results.php?",
		    			data: {
		    				table: Drupal.settings.dataDisplaySource,
		    				fields: Drupal.settings.dataDisplayFields.split(","),
		    				filterField: Drupal.settings.dataFilterField,
		    				filterValue: mapSettings.selectedLocation,
		    				startDate: startDatePicker.getDate(),
		    				endDate: endDatePicker.addDaysToDate(1) // adds 1 day to selected end date
		    			}
		    		})
	    		},
	    		setData: function() {
	    			$.when(chartRequest.fetchData()).done(function(data) {
		   				var response = JSON.parse(data);
		   				if (response) {
		   					chartRequest.data = response;
		   					chart = new Chart(chartOptions, chartRequest.data);
		   				}
		   			})
	    		}
	    	}
	    	chartRequest.setData();

	    	// configure chart2 options
	    	var chartOptions2 = {
	    		type: +Drupal.settings.dataDisplayType2,
	    		container: "chart2",
	    		fields: Drupal.settings.dataDisplayFields2.split(","),
	    		fieldLabels: Drupal.settings.dataDisplayFieldLabels2.split(","),
	    		fieldTypes: Drupal.settings.dataDisplayFieldTypes2.split(",")
				};

	    	// request chart2 data from database
	    	var chartRequest2 = {
	    		fetchData: function() {
	    			return $.ajax({
	    				url: "/sites/all/themes/lrebootstrap/scripts/php/view/results.php?",
		    			data: {
		    				table: Drupal.settings.dataDisplaySource2,
		    				fields: Drupal.settings.dataDisplayFields2.split(","),
		    				filterField: Drupal.settings.dataFilterField2,
		    				filterValue: mapSettings.selectedLocation,
		    				startDate: startDatePicker.getDate(),
		    				endDate: endDatePicker.addDaysToDate(1), // adds 1 day to selected end date
		    				windowFunction: true,
		    				windowSumField: "daily_volume_bbl",
		    				windowPartitionField: "location_ndx",
		    				windowOrderField: "date_time",
		    			}
		    		})
	    		},
	    		setData: function() {
	    			$.when(chartRequest2.fetchData()).done(function(data) {
		   				var response = JSON.parse(data);
		   				if (response) {
		   					chartRequest2.data = response;
		   					chart2 = new Chart(chartOptions2, chartRequest2.data);
		   				}
		   			})
	    		}
	    	}
	    	chartRequest2.setData();

	    	// configure chart3 options
	    	var chartOptions3 = {
	    		type: +Drupal.settings.dataDisplayType3,
	    		container: "chart3",
	    		fields: Drupal.settings.dataDisplayFields3.split(","),
	    		fieldLabels: Drupal.settings.dataDisplayFieldLabels3.split(","),
	    		fieldTypes: Drupal.settings.dataDisplayFieldTypes3.split(",")
	    	};

	    	// request chart3 data from database
	    	var chartRequest3 = {
	    		fetchData: function() {
	    			return $.ajax({
	    				url: "/sites/all/themes/lrebootstrap/scripts/php/view/results.php?",
		    			data: {
		    				table: Drupal.settings.dataDisplaySource3,
		    				fields: Drupal.settings.dataDisplayFields3.split(","),
		    				filterField: Drupal.settings.dataFilterField3,
		    				filterValue: mapSettings.selectedLocation,
		    				startDate: startDatePicker.getDate(),
		    				endDate: endDatePicker.addDaysToDate(1) // adds 1 day to selected end date
		    			}
		    		})
	    		},
	    		setData: function() {
	    			$.when(chartRequest3.fetchData()).done(function(data) {
		   				var response = JSON.parse(data);
		   				if (response) {
		   					chartRequest3.data = response;
		   					chart3 = new Chart(chartOptions3, chartRequest3.data);
		   				}
		   			})
	    		}
	    	}
	    	chartRequest3.setData();

	    	// configure chart3 options
	    	chartOptions4 = {
	    		type: 5,
	    		container: "gauge-chart",
	    		fields: ["descrip","hrs_since_service"],
	    		fieldLabels: ["Well Name", "Hours Since Last Service"],
	    		fieldTypes: ["string", "number"],
	    		gaugeRanges: []
	    	};

	    	// request chart4 data from database
	    	var chartRequest4 = {
	    		fetchData: function() {
	    			return $.ajax({
	    				url: "/sites/all/themes/lrebootstrap/scripts/php/view/results.php?",
		    			data: {
		    				table: "dashboard.service_hours_since_last",
		    				fields: ["descrip","hrs_since_service"],
		    				filterField: "location_ndx",
		    				filterValue: mapSettings.selectedLocation
		    			}
		    		})
	    		},
	    		setData: function() {
	    			$.when(chartRequest4.fetchData()).done(function(data) {
		   				var response = JSON.parse(data);
		   				if (response) {
		   					chartRequest4.data = response;
		   					maintenanceSettings();
		   					chart4 = new Chart(chartOptions4, chartRequest4.data);
		   				}
		   			})
	    		}
	    	}
	    	chartRequest4.setData();
	    	updateDataTitles();
			maintenanceSettings();
		}

		function createDatePickers() {
			var startDatePickerOptions = {
				container: "#map-start-date-picker"
			}
			var endDatePickerOptions = {
				container: "#map-end-date-picker"
			}

			startDatePicker = new Datepicker(startDatePickerOptions);
			startDatePicker.setDate(10);

			endDatePicker = new Datepicker(endDatePickerOptions);
			endDatePicker.setDate();
		} // createDatePickers()
		createDatePickers();

		// create location dropdown menu
		var locationDropdownOptions = {
			container: "#data-display-container-mobile-1 #map-location-select", // define HTML id/class of select element that will house dropdown options
			valueIndex: 0, // set index aka position of column that contains values
			textIndex: 1, // set index aka position of column that contains text to display
			onChange: function() { // define behavior when selected value changes
				mapSettings.selectedLocation = locationDropdown.getValue();
				mapSettings.selectedLocationName = locationDropdown.getText();
			}
		}

		var locationDropdownRequest = {
			url: "/sites/all/themes/lrebootstrap/scripts/php/view/index.php",
			data: {
				table: "mapbox.locations",
				fields: ['location_ndx', 'location_desc'],
				drupal_user_switch: false,
				drupal_userid: 1
			}
		}

		var locationDropdownOptions2 = {
			container: "#meta-data-display-container-1 #map-location-select", // define HTML id/class of select element that will house dropdown options
			valueIndex: 0, // set index aka position of column that contains values
			textIndex: 1, // set index aka position of column that contains text to display
			onChange: function() { // define behavior when selected value changes
				mapSettings.selectedLocation = locationDropdown2.getValue();
				mapSettings.selectedLocationName = locationDropdown2.getText();
			}
		}

		var locationDropdown = new Dropdown(locationDropdownOptions,locationDropdownRequest);
		var locationDropdown2 = new Dropdown(locationDropdownOptions2,locationDropdownRequest);

		// Mapbox API access token, each Mapbox account has their own
		mapboxgl.accessToken = 'pk.eyJ1Ijoid2lua2xlci1kYXNoYm9hcmQiLCJhIjoiY2o4MjM2ZTFhNTBmODJ3bzN0Mml6ZTJvbSJ9.6990gDMQ61xhUkEcdo_1mA';
	    
		// create Mapbox map object, 
		// passing in options from mapSettings object
	    var map = new mapboxgl.Map({
	        container: 'map',
	        style: mapSettings.style,
	        zoom: mapSettings.zoomLevel,
	        center: [mapSettings.center[1], mapSettings.center[0]]
	    });

	    // Add zoom and rotation controls to the map if enabled
	    if (mapSettings.zoomControl == 1) {	
	      map.addControl(new mapboxgl.NavigationControl(), 'top-left');
	    }

	    // resize map to fit container after map loads
	    map.on('load', function() {
	      	map.resize();

	      	// initialize data source
	      	map.addSource("station_locations", {
	      		type: "geojson",
	      		data: "/sites/all/themes/lrebootstrap/scripts/php/spatial/index.php?geotable=mapbox.locations_detailed&geomfield=geom&srid=4326&fields=location_ndx,location_desc,most_recent_timestamp,most_recent_value"
	      	})

	      	// Add USGS Stream Gages from station_locations source
			map.addLayer({
   				"id":"wells",
       			"type": "circle",
       			"source": "station_locations",
       			"paint": {
         			"circle-radius": 6,
         			"circle-color": "#5bc0de",
         			"circle-stroke-width": 2,
         			"circle-stroke-color": "#333"
   				}
     		})

     		locationDropdown.setValue(mapSettings.selectedLocation);
     		locationDropdown2.setValue(mapSettings.selectedLocation);
    	}) // map.on("load")


	    // define what happens when user clicks on the map
	    // i.e. open popup
    	map.on('click', function (e) {
   			// check if instance of mapboxgl.Popup exists, if so remove all popups
       		if (popup instanceof mapboxgl.Popup) {
           		popup.remove();
       		} 
			
			// evaluate map where user has clicked, return list of features
			var features = map.queryRenderedFeatures(e.point, { layers: ['wells'] });
			
			// if no features exist, exit
			if (!features.length) {
            	return;
         	}

         	// set feature equal to first returned feature
	        var feature = features[0];
	        mapSettings.selectedLocation = feature.properties.location_ndx;
	        mapSettings.selectedLocationName = feature.properties.location_desc;

         	// create popup, set popup content, add popup to map
     		createPopup(feature);

     		// set dropdown value to match selected location
     		locationDropdown.setValue(mapSettings.selectedLocation);
     		locationDropdown2.setValue(mapSettings.selectedLocation);

     		// if data pane hidden by default, shrink map and show data pane
	    	$( '#map' ).css('height', ($(window).height())*.5);

     		// create chart based on selected location
     		createChart();


	    }); // map.on("click")

    	// determine if data pane should be hidden or visible by default
    	if (dataSettings.visiblity == 1) {
    		$( '#map' ).css('height', ($(window).height())*.5);
    	} else {
			$( '#map' ).css('height', $(window).height());
    	}

    	createChart();

    	/*$(window).resize(function() {
   			$('#map').css('height', $(window).height());
     	});*/

	})
})(jQuery);
