<?php
/**
 * @file
 * Default theme implementation to display a node.
 *
 * Available variables:
 * - $title: the (sanitized) title of the node.
 * - $content: An array of node items. Use render($content) to print them all,
 *   or print a subset such as render($content['field_example']). Use
 *   hide($content['field_example']) to temporarily suppress the printing of a
 *   given element.
 * - $user_picture: The node author's picture from user-picture.tpl.php.
 * - $date: Formatted creation date. Preprocess functions can reformat it by
 *   calling format_date() with the desired parameters on the $created variable.
 * - $name: Themed username of node author output from theme_username().
 * - $node_url: Direct URL of the current node.
 * - $display_submitted: Whether submission information should be displayed.
 * - $submitted: Submission information created from $name and $date during
 *   template_preprocess_node().
 * - $classes: String of classes that can be used to style contextually through
 *   CSS. It can be manipulated through the variable $classes_array from
 *   preprocess functions. The default values can be one or more of the
 *   following:
 *   - node: The current template type; for example, "theming hook".
 *   - node-[type]: The current node type. For example, if the node is a
 *     "Blog entry" it would result in "node-blog". Note that the machine
 *     name will often be in a short form of the human readable label.
 *   - node-teaser: Nodes in teaser form.
 *   - node-preview: Nodes in preview mode.
 *   The following are controlled through the node publishing options.
 *   - node-promoted: Nodes promoted to the front page.
 *   - node-sticky: Nodes ordered above other non-sticky nodes in teaser
 *     listings.
 *   - node-unpublished: Unpublished nodes visible only to administrators.
 * - $title_prefix (array): An array containing additional output populated by
 *   modules, intended to be displayed in front of the main title tag that
 *   appears in the template.
 * - $title_suffix (array): An array containing additional output populated by
 *   modules, intended to be displayed after the main title tag that appears in
 *   the template.
 *
 * Other variables:
 * - $node: Full node object. Contains data that may not be safe.
 * - $type: Node type; for example, story, page, blog, etc.
 * - $comment_count: Number of comments attached to the node.
 * - $uid: User ID of the node author.
 * - $created: Time the node was published formatted in Unix timestamp.
 * - $classes_array: Array of html class attribute values. It is flattened
 *   into a string within the variable $classes.
 * - $zebra: Outputs either "even" or "odd". Useful for zebra striping in
 *   teaser listings.
 * - $id: Position of the node. Increments each time it's output.
 *
 * Node status variables:
 * - $view_mode: View mode; for example, "full", "teaser".
 * - $teaser: Flag for the teaser state (shortcut for $view_mode == 'teaser').
 * - $page: Flag for the full page state.
 * - $promote: Flag for front page promotion state.
 * - $sticky: Flags for sticky post setting.
 * - $status: Flag for published status.
 * - $comment: State of comment settings for the node.
 * - $readmore: Flags true if the teaser content of the node cannot hold the
 *   main body content.
 * - $is_front: Flags true when presented in the front page.
 * - $logged_in: Flags true when the current user is a logged-in member.
 * - $is_admin: Flags true when the current user is an administrator.
 *
 * Field variables: for each field instance attached to the node a corresponding
 * variable is defined; for example, $node->body becomes $body. When needing to
 * access a field's raw values, developers/themers are strongly encouraged to
 * use these variables. Otherwise they will have to explicitly specify the
 * desired field language; for example, $node->body['en'], thus overriding any
 * language negotiation rule that was previously applied.
 *
 * @see template_preprocess()
 * @see template_preprocess_node()
 * @see template_process()
 *
 * @ingroup templates
 */
?>

<!-- Export content type fields as javascript variables -->
<?php
/* Export Map Settings */
drupal_add_js(array('mapStyle' => $field_map_style['und'][0]["safe_value"]), 'setting');
drupal_add_js(array('mapCenter' => [$field_map_center['und'][0]['lat'], $field_map_center['und'][0]['lon']]), 'setting');
drupal_add_js(array('mapZoomLevel' => $field_map_zoom_level['und'][0]['value']), 'setting');
drupal_add_js(array('mapZoomControl' => $field_enable_zoom_control['und'][0]['value']), 'setting');

/* Export Data Pane 1 Settings */
if (!empty($field_dropdown1_source_view)) { drupal_add_js(array('dataSource1' => $field_dropdown1_source_view["und"][0]["safe_value"]), 'setting'); };
drupal_add_js(array('dataFilterField' => $field_de_filter_field['und'][0]['safe_value']), 'setting');
drupal_add_js(array('dataDisplayTitle1' => $field_de_display_title['und'][0]['value']), 'setting');
drupal_add_js(array('dataDisplaySource' => $field_de_source_view['und'][0]['safe_value']), 'setting');
drupal_add_js(array('dataDisplayFields' => $field_de_source_fields['und'][0]['safe_value']), 'setting');
drupal_add_js(array('dataDisplayFieldLabels' => $field_de_source_field_labels['und'][0]['safe_value']), 'setting');
drupal_add_js(array('dataDisplayFieldTypes' => $field_de_source_field_types['und'][0]['safe_value']), 'setting');
drupal_add_js(array('dataDisplayType' => $field_de_display_type['und'][0]['value']), 'setting');

/* Export Data Pane 2 Settings */
drupal_add_js(array('dataFilterField2' => $field_de_filter_field2['und'][0]['safe_value']), 'setting');
drupal_add_js(array('dataDisplayTitle2' => $field_de_display_title2['und'][0]['value']), 'setting');
drupal_add_js(array('dataDisplaySource2' => $field_de_source_view2['und'][0]['safe_value']), 'setting');
drupal_add_js(array('dataDisplayFieldLabels2' => $field_de_source_field_labels2['und'][0]['safe_value']), 'setting');
drupal_add_js(array('dataDisplayFieldTypes2' => $field_de_source_field_types2['und'][0]['safe_value']), 'setting');
drupal_add_js(array('dataDisplayFields2' => $field_de_source_fields2['und'][0]['safe_value']), 'setting');
drupal_add_js(array('dataDisplayType2' => $field_de_display_type2['und'][0]['value']), 'setting');

/* Export Data Pane 3 Settings */
drupal_add_js(array('dataFilterField3' => $field_de_filter_field3['und'][0]['safe_value']), 'setting');
drupal_add_js(array('dataDisplayTitle3' => $field_de_display_title3['und'][0]['value']), 'setting');
drupal_add_js(array('dataDisplaySource3' => $field_de_source_view3['und'][0]['safe_value']), 'setting');
drupal_add_js(array('dataDisplayFieldLabels3' => $field_de_source_field_labels3['und'][0]['safe_value']), 'setting');
drupal_add_js(array('dataDisplayFieldTypes3' => $field_de_source_field_types3['und'][0]['safe_value']), 'setting');
drupal_add_js(array('dataDisplayFields3' => $field_de_source_fields3['und'][0]['safe_value']), 'setting');
drupal_add_js(array('dataDisplayType3' => $field_de_display_type3['und'][0]['value']), 'setting');

/* Export Data Pane Visibility Setting */
drupal_add_js(array('dataDisplayVisibility' => $field_de_display_state['und'][0]['value']), 'setting');
?>

<!-- Map filters sidebar toggle -->
<!-- Only visible on smaller screens -->
<div class="left-sidebar-toggle">
  <div class="sidebar-toggle-title">View filters</div>
</div>

<!-- Backdrop on smaller screens that makes background opaque when sidebar is open -->
<div class="backdrop" style="display: none;"></div>

<!-- Map filters sidebar -->
<section class="left-sidebar">
  <div id="filters-header-background" class="col-xs-12 layout-padding">
    <i class="material-icons md-36 pull-left text-white">query_builder</i>
    <h4 class="filters-header pull-left">Map Filters</h4>
  </div>
  <div class="clearfix"></div>
  <div class="btn-wrapper layout-padding">
    <button class="btn btn-primary hide-sidebar-left-btn">Hide filters</button>
  </div>

  <div class="custom-form-group layout-margin">
    <div class="form-label-wrapper">
      <i class="material-icons md-24 text-slate">date_range</i> 
      <label class="text-slate"><?php print $field_dropdown1_label["und"][0]["safe_value"]; ?></label>
    </div>
    <?php if ($field_dropdown1_type["und"][0]["value"] == 1): ?>
    <select id="map-dropdown1" class="form-control"></select>
    <?php elseif ($field_dropdown1_type["und"][0]["value"] == 3): ?>
    <input id="map-start-date-picker" type="date" class="form-control" />
    <?php endif; ?>
  </div>

  <div class="custom-form-group layout-margin">
    <div class="form-label-wrapper">
      <i class="material-icons md-24 text-slate">date_range</i>  
      <label class="text-slate"><?php print $field_dropdown2_label["und"][0]["safe_value"]; ?></label>
    </div>
    <?php if ($field_dropdown2_type["und"][0]["value"] == 1): ?>
    <select id="map-dropdown1" class="form-control"></select>
    <?php elseif ($field_dropdown2_type["und"][0]["value"] == 3): ?>
    <input id="map-end-date-picker" type="date" class="form-control"  />
    <?php endif; ?>
  </div>
  <div class="col-xs-12">
    <button id="btn-date-filter" class="btn btn-filter">Apply filters</button>
  </div>
  <div class="col-xs-12 divider"></div>
  <div class="col-xs-12 btn-wrapper layout-padding">
    <a href="<?php print $field_sidebar_download_link["und"][0]["safe_value"] ?>" target="_blank"><button id="btn-download" class="btn btn-filter">Download Data</button></a>
  </div>
</section>

<!-- Main page content including map and graphs -->
<section class="main-content">
  <article id="node-<?php print $node->nid; ?>" class="<?php print $classes; ?> clearfix"<?php print $attributes; ?>>
    <?php if ((!$page && !empty($title)) || !empty($title_prefix) || !empty($title_suffix) || $display_submitted): ?>
    <header>
      <?php print render($title_prefix); ?>
      <?php if (!$page && !empty($title)): ?>
      <h2<?php print $title_attributes; ?>><a href="<?php print $node_url; ?>"><?php print $title; ?></a></h2>
      <?php endif; ?>
      <?php print render($title_suffix); ?>
      <?php if ($display_submitted): ?>
      <span class="submitted">
        <?php print $user_picture; ?>
        <?php print $submitted; ?>
      </span>
      <?php endif; ?>
    </header>
    <?php endif; ?>
    
    <!-- Mapbox map container -->
    <div id="map"></div>
    <div class="border-top"></div>
    <!-- Data Display Section -->
    <section id="data-pane">
      <section id="data-display-section">
        <div id="data-display-container-mobile-1" class="meta-data-display-container">
          <div class="layout-padding">
            <!-- <i class="material-icons md-36 text-green pull-left">filter_list</i> -->
            <h4 class="pull-left meta-data-display-title">Select a location</h4>
            <div class="clearfix"></div>
            <select id="map-location-select" class="form-control"></select>
            <div class="col-xs-12">
              <button class="btn btn-filter btn-loc-filter">Select location</button>
            </div>
            <div class="clearfix"></div>
          </div>
        </div>
        <div id="data-display-container-1" class="data-display-container data-display-container-first">
          <div class="layout-padding">
            <i class="material-icons md-36 text-blue pull-left">assessment</i>
            <h3 class="pull-left data-display-title"><?php print $field_de_display_title['und'][0]['value']; ?></h3>
          </div>
          <button class="btn btn-hide pull-right">Hide Pane</button>
          <div class="clearfix"></div>
          <div id="chart" class="layout-margin"></div>
          <div class="layout-margin">Pump Depth for this well is 400 ft below the ground surface</div>
        </div>
        <div class="clearfix"></div>
        <div id="data-display-container-2" class="data-display-container">
          <div class="layout-padding">
            <i class="material-icons md-36 text-blue pull-left">assessment</i>
            <h3 class="pull-left data-display-title"><?php print $field_de_display_title2['und'][0]['value']; ?></h3>
          </div>
          <div class="clearfix"></div>
          <div id="chart2" class="layout-margin"></div>
        </div>
        <div id="data-display-container-3" class="data-display-container">
          <div class="layout-padding">
            <i class="material-icons md-36 text-blue pull-left">assessment</i>
            <h3 class="pull-left data-display-title"><?php print $field_de_display_title3['und'][0]['value']; ?></h3>
          </div>
          <div class="clearfix"></div>
          <div id="chart3" class="layout-margin"></div>
        </div>
      </section>

      <section id="meta-data-display-section">
        <div id="meta-data-display-container-1" class="meta-data-display-container">
          <div class="layout-padding">
            <!-- <i class="material-icons md-36 text-green pull-left">filter_list</i> -->
            <h4 class="pull-left meta-data-display-title">Select a location</h4>
            <div class="clearfix"></div>
            <select id="map-location-select" class="form-control"></select>
            <div class="col-xs-12">
              <button class="btn btn-filter btn-loc-filter">Select location</button>
            </div>
            <div class="clearfix"></div>
          </div>
        </div>
        <div id="meta-data-display-container-2" class="meta-data-display-container">
          <div class="layout-padding">
            <!-- <i class="material-icons md-36 text-green pull-left">filter_list</i> -->
            <h4 class="meta-data-display-title">Hours Pumped Since Last Service</h4>
            <div id="gauge-chart"></div>
            <h4 id="service-log-title" class="meta-data-display-title">Last Service Log Entries for Well 2</h4>
            <div class="clearfix"></div>
            <table class="table table-bordered" id="service-log-table">
            </table>
            <!-- <a href="http://winkler.lrecloud.com/maintenance"><button class="btn btn-filter">Open Maintenance Module</button></a> -->
            <h4 class="meta-data-display-title">Maintenance Settings</h4>
            <ul id="maintenance-settings-list"></ul>
          </div>
          <div class="clearfix"></div>
        </div>
      </section>
    </section> <!-- #data-pane -->
    <!-- Load moment.js -->
    <script src="https://cdnjs.cloudflare.com/ajax/libs/moment.js/2.19.0/moment.min.js"></script>
    <!-- Load Mercury forms library -->
    <script src="/sites/all/themes/lrebootstrap/scripts/js/mercury.latest.min.js"></script>
    <!-- load GVIZ -->
    <script type="text/javascript" src="https://www.gstatic.com/charts/loader.js"></script>
    <!-- Load Mapbox CSS and Javascript libraries -->  
    <script src='https://api.tiles.mapbox.com/mapbox-gl-js/v0.39.1/mapbox-gl.js'></script>
    <link href='https://api.tiles.mapbox.com/mapbox-gl-js/v0.39.1/mapbox-gl.css' rel='stylesheet' />

    <script src="/sites/all/themes/lrebootstrap/scripts/js/map.js"></script>
    <script src="/sites/all/themes/lrebootstrap/scripts/js/charts.js"></script>

    <!-- LRE Content Type Specific Styles -->
    <link rel="stylesheet" href="/sites/all/themes/lrebootstrap/css/sidebar.css" ?>
    <link rel="stylesheet" href="/sites/all/themes/lrebootstrap/css/data-pane.css" ?>
    <link rel="stylesheet" href="/sites/all/themes/lrebootstrap/css/map.css" ?>


    <?php
      // Hide comments, tags, and links now so that we can render them later.
      hide($content['comments']);
      hide($content['links']);
      hide($content['field_tags']);
      print render($content);
    ?>
    <?php
      // Only display the wrapper div if there are tags or links.
      $field_tags = render($content['field_tags']);
      $links = render($content['links']);
      if ($field_tags || $links):
    ?>
     <footer>
       <?php print $field_tags; ?>
       <?php print $links; ?>
    </footer>
    <?php endif; ?>
    <?php print render($content['comments']); ?>
  </article>
</<section>

